<?php

namespace app\controllers;

use Yii;
use app\models\Configurations;
use app\models\Store;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

//ini_set('session.cookie_samesite', 'None');
//session_set_cookie_params(['samesite' => 'None']);

/**
 * ConfigurationController implements the CRUD actions for Configurations model.
 */
class ConfigurationController extends Controller {

    public function beforeAction($action) {

        $this->enableCsrfValidation = false;
        return parent::beforeAction($action);
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Load Configuration model according to Store.
     * @return mixed
     */
    public function actionIndex() {

        $session = Yii::$app->session;
        $session->open();
        if (!empty($session->get('shop'))) {
            $shop = $session->get('shop');
        } else {
            $shop_name = explode('?', Yii::$app->request->url);
            $shop = $shop_name[1];
        }

        $config = [
            'enable_square' => '', 'live_application_id' => '', 'live_access_token' => '', 'live_location_id' => '',
            'sandbox' => '', 'sandbox_application_id' => '', 'sandbox_access_token' => '', 'sandbox_location_id' => ''
        ];

        $meta_key = $shop . '_square_detail';
        $model = Configurations::find()->where(['meta_key' => $meta_key])->one();
        if (isset($model)) {
            $config = unserialize($model->meta_value);
        }
        $config['shop'] = $shop;

        return $this->render('index', ['config' => $config]);
    }

    /*
     * Save configurations in database by creating or updating
     */

    public function actionSetConfig() {

        if (!Yii::$app->request->isPost) {
            return $this->redirect('index');
        }

        $session = Yii::$app->session;
        $shop = !empty($session->get('shop')) ? $session->get('shop') : Yii::$app->request->post('shop');

        $data = array(
            "enable_square" => Yii::$app->request->post('enable_square'),
            "sandbox" => Yii::$app->request->post('sandbox'),
            "sandbox_application_id" => Yii::$app->request->post('sandbox_application_id'),
            "sandbox_access_token" => Yii::$app->request->post('sandbox_access_token'),
            "sandbox_location_id" => Yii::$app->request->post('sandbox_location_id'),
            "live_application_id" => Yii::$app->request->post('live_application_id'),
            "live_access_token" => Yii::$app->request->post('live_access_token'),
            "live_location_id" => Yii::$app->request->post('live_location_id'),
        );
        $meta_value = serialize($data);
        $meta_key = $shop . '_square_detail';
        $Store = Store::find()->where(['name' => $shop])->one();

        $model = new Configurations();
        $model->meta_key = $meta_key;
        $model->meta_value = $meta_value;
        $model->store_id = $Store->id;
        $Setting = Configurations::find()->where(['meta_key' => $meta_key])->one();
        if ($Setting) {
            $model->updateAll(['meta_value' => $meta_value], ['like', 'meta_key', $shop . '_square_detail']);
        } else {
            $model->save(false);
        }
//        echo "<pre>"; print_r($model); die('wait');
        return $this->redirect('index?'.$shop);
    }

    /**
     * Finds the Configurations model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Configurations the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id) {
        if (($model = Configurations::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function actionTest() {
        $session = Yii::$app->session;
        $session->open();
        $session->set('shop', $shop);
    }

}
