<?php

namespace app\controllers;

use Yii;
use app\models\Orders;
use app\models\OrdersSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\CartDetails;

/**
 * OrderController implements the CRUD actions for Orders model.
 */
class OrderController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    public function beforeAction($action) {
        $this->enableCsrfValidation = false;
        return parent::beforeAction($action);
    }

    public function actionRefersion()
    {
        ignore_user_abort(true);
        ob_start();
        $serverProtocole = filter_input(INPUT_SERVER, 'SERVER_PROTOCOL', FILTER_SANITIZE_STRING);
        header($serverProtocole . ' 200 OK');
        header('Content-Encoding: none');
        header('Content-Length: ' . ob_get_length());
        header('Connection: close');
        ob_end_flush();
        ob_flush();
        flush();

        $webhookContent = '';
        $webhook = fopen('php://input', 'rb');
	while (!feof($webhook)) {
	    $webhookContent .= fread($webhook, 4096);
	}

        $logs = fopen("hooks_log.txt", "a") or die("Unable to open file!");
        fwrite($logs, "\n\n [".date("Y-m-d H:i:s A")."] Order: \n\n");
//        fwrite($logs, "\n\n $webhookContent\n\n");

	$_order = json_decode($webhookContent, true);
        $order_id = $_order['id'];
        fwrite($logs, "order_id: $order_id \n\n");

        $model = CartDetails::find()->where(['order_id' => $order_id])->one();

        if($model !== null) {

            $currency = $_order['presentment_currency'];
            fwrite($logs, "Shop: $currency \n\n");

            $items = [];
            foreach($_order['line_items'] as $item) {
                $items[] = ["price" => floatval($item['price']), "quantity" => intval($item['quantity']), "sku" => $item['sku'], "name" => $item['title']];
            }
//          fwrite($logs, "Items: ". json_encode($items));
            
            $customer = $_order['customer'];
//          fwrite($logs, "customer: ". json_encode($customer));

            $cart_id = $model->cart_token;
            $params = [
                "cart_id" => $cart_id,
                "order_id" => strval($order_id),
//                "subscription_id" => strval($order_id),
//                "is_subscription" => true,
                "shipping" => !empty($_order['shipping_lines']) ? floatval($_order['shipping_lines'][0]['price']) : 0,
                "tax" => floatval($_order['total_tax']),
                "discount" => floatval($_order['total_discounts']),
                "discount_code" => !empty($_order['discount_applications']) ? $_order['discount_applications'][0]['code'] : "",
                "currency_code" => $currency,
                "customer" => [
                    "first_name" => $customer['first_name'],
                    "last_name" => $customer['last_name'],
                    "email" => $customer['email']
                ],
                "items" => $items
            ];

            try {
                // Convert array into JSON
                $json_data = json_encode($params);
                fwrite($logs, "json_data = $json_data");
                $key = Yii::$app->params['REFERSION_KEY'];
                $secret = Yii::$app->params['REFERSION_SECRET'];

                // The URL that you are posting to
                $url = 'https://inbound-webhooks.refersion.com/tracker/orders/paid';

                // Start cURL
                $curl = curl_init($url);
                
                // Verify that our SSL is active (for added security)
                curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

                // Send as a POST
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');

                // The JSON data that you have already compiled
                curl_setopt($curl, CURLOPT_POSTFIELDS, $json_data);

                // Return the response
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

                // Set headers to be JSON-friendly
                curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                    'Content-Type: application/json',
                    'Content-Length: ' . strlen($json_data),
                    'Refersion-Public-Key: '. $key,
                    'Refersion-Secret-Key: '. $secret
                ));

                // Seconds (30) before giving up
                curl_setopt($curl, CURLOPT_TIMEOUT, 90);
                curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 90);

                // Execute post, capture response (if any) and status code
                $result = curl_exec($curl);
//                $errno = curl_errno($curl);
//                $error = curl_error($curl);
                $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                // Close connection
                curl_close($curl);

                fwrite($logs, "\n\n status: $status". json_encode($result));
                return true;
            } catch (Exception $e) {
                fwrite($logs, "\n Exception from Refersion: ".$e->getMessage());
//                echo 'Exception when calling RefundsApi->refundPayment: ', $e->getMessage(), PHP_EOL;
                return true;
            }
        }
        return true;
    }

    /**
     * Lists all Orders models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new OrdersSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single Orders model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new Orders model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Orders();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Orders model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Orders model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Orders model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Orders the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Orders::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
