<?php

namespace app\controllers;

use Yii;
use app\models\Store;
use yii\web\Controller;
use yii\filters\VerbFilter;
use app\models\ShopifyClient;

/**
 * StoreController implements the CRUD actions for Store model.
 */
class StoreController extends Controller {

    public function beforeAction($action) {
        $this->enableCsrfValidation = false;
        return parent::beforeAction($action);
    }

    /**
     * {@inheritdoc}
     */
    public function behaviors() {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    /**
     * Lists all Store models.
     * @return mixed
     */
    public function actionInstall() {

        $shop = Yii::$app->request->get('shop');
        $code = Yii::$app->request->get('code');
		
        $token = '';
        $storeExits = Store::find()->where(['name' => $shop])->one();
        $key = Yii::$app->params['APP_KEY'];
        $secret = Yii::$app->params['SECRET_KEY'];
        if (!$storeExits) {

            $sc = new ShopifyClient($shop, "", $key, $secret);

            if (!empty($code)):
                $token = $sc->getAccessToken($code);
            endif;

            if (empty($token)) {
               return $this->redirect($sc->getAuthorizeUrl(Yii::$app->params['SHOPIFY_SCOPE'], Yii::$app->params['REDIRECT_URI']));
                exit;
            } else  if (!empty($token)) {
                $StoreInsert = new Store();
                $StoreInsert->token = $token;
                $StoreInsert->name = $shop;
                $StoreInsert->code = $code;
                $StoreInsert->status = 1;

                if ($StoreInsert->save(false)) {

                    $sc = new ShopifyClient($shop, $token, $key, $secret);
                    $this->addWebHooks($sc, $shop);
                    $this->addScriptTag($sc);

                    $cookies = Yii::$app->response->cookies;
                    $cookies->add(new \yii\web\Cookie([
                        'name' => 'cross-site-cookie',
                        'value' => 'bar',
                        'sameSite' => 'None',
                        'secure' => true,
                    ]));
                    $session = Yii::$app->session;
                    $session->open();
                    $session->set('shop', $shop);
                    // $this->redirect('index');
                     return $this->redirect('https://' . $shop . '/admin/apps/');
                    exit;
                } else {
                    Yii::$app->session->setFlash('error', json_encode($StoreInsert->getErrors()));
                    return $this->redirect('https://' . $shop . '/admin/apps/');
                }
            }
        } else {
            $sc = new ShopifyClient($shop, $storeExits->token, $key, $secret);
            //$this->addWebHooks($sc, $shop);
            $cookies = Yii::$app->response->cookies;
            $cookies->add(new \yii\web\Cookie([
                'name' => 'cross-site-cookie',
                'value' => 'bar',
                'sameSite' => 'None',
                'secure' => true,
            ]));
            $session = Yii::$app->session;
            $session->open();
            $session->set('shop', $shop);
            //echo '<pre>';print_r($storeExits); die; 
            $this->redirect(['configuration/index?store=' . $storeExits->name]);
			
        }
    }

    /**
     * Un-install hook to delete store data from DB
     * @return mixed
     */
    public function actionUninstall() {

        $shop = Yii::$app->request->get('shop');

//        $SHOPIFY_APP_SECRET = Yii::$app->params['SECRET_KEY'];
//        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
//        $data = file_get_contents('php://input');
//        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $SHOPIFY_APP_SECRET, true));
//        $verified = hash_equals($hmac_header, $calculated_hmac);
//        $myfile = fopen("hooks_log.txt", "a") or die("Unable to open file!");
//        $txt = print_r(var_export($verified, true), true);
//        fwrite($myfile, $txt);
//        fclose($myfile);
//        if ($verified): //If Hook is Verified Fetch the Webhook content and Save Data in Table
        $webhookContentData = "";
        $webhook = fopen('php://input', 'rb');
        while (!feof($webhook)):
            $webhookContentData .= fread($webhook, 4096);
        endwhile;
        fclose($webhook);
        $webhookContent = json_decode($webhookContentData);

        $myfile = fopen("hooks_log.txt", "a") or die("Unable to open file!");
        $txt = print_r($webhookContent, true);
        fwrite($myfile, $txt);
        fwrite($myfile, 'shop = ' . $shop);
        fclose($myfile);

        $Store = Store::find()->where(['name' => $shop])->one();

        if (!empty($Store)){
			\Yii::$app
				->db
				->createCommand()
				->delete('store', ['name' => $shop])
				->execute();
		}
           


    }

    /*
     * Create Shopify webhooks
     */

    protected function addWebHooks($sc, $shop) {

        $base_url = Yii::$app->params['BASE'];
        $version = Yii::$app->params['SHOPIFY_API_VERSION'];
        $hooks = array();

        try {
            $hooks_tag = $sc->call('GET', "/admin/api/$version/webhooks.json");
			if(!empty($hooks_tag)){
				foreach ($hooks_tag as $exist_hooks) {
				//	if(is_array($exist_hooks))
                $hooks[] = $exist_hooks['topic'];
				}
			}
            
            if (!in_array("app/uninstalled", $hooks)) {
                $uninstall_hook = array(
                    "webhook" => array(
                        "topic" => "app/uninstalled",
                        "address" => $base_url . "index.php/store/uninstall?shop=" . $shop . "",
                        "format" => "json"
                    )
                );
                $hooks_tag = $sc->call('POST', "/admin/api/$version/webhooks.json", $uninstall_hook);
            }
//            if (!in_array("orders/create", $hooks)) {
//                $order_create_hook = [
//                    "webhook" => [
//                        "topic" => "orders/create",
//                        "address" => $base_url . "order/refersion?shop=" . $shop,
//                        "format" => "json"
//                    ]
//                ];
//                $hooks_tag = $sc->call('POST', "/admin/api/$version/webhooks.json", $order_create_hook);
//            }
            if (!in_array("refunds/create", $hooks)) {
                $refund_create_hook = [
                    "webhook" => [
                        "topic" => "refunds/create",
                        "address" => $base_url . "refund/sync?shop=" . $shop,
                        "format" => "json"
                    ]
                ];
                $hooks_tag = $sc->call('POST', "/admin/api/$version/webhooks.json", $refund_create_hook);
            }
        } catch (Exception $exc) {
            echo "<pre>";
            print_r($exc);
            die('hook');
        }
    }

    /*
     * Add Script Tags in shopify 
     */

    protected function addScriptTag($sc) {

        $base_url = Yii::$app->params['BASE'];
        $version = Yii::$app->params['SHOPIFY_API_VERSION'];

        $checkout_script = array(
            "script_tag" => array(
                "event" => "onload",
                "src" => $base_url . "js/checkout.js"
            )
        );

        $sc->call('POST', "/admin/api/$version/script_tags.json", $checkout_script);
    }

    public function actionTest() {
        $base_url = Yii::$app->params['BASE_URL'];
        $version = Yii::$app->params['SHOPIFY_API_VERSION'];
        $shop = '';
        $token = '';
        $key = '';
        $secret = '';

        $sc = new ShopifyClient($shop, $token, $key, $secret);
        $delete = $sc->call('DELETE', "/admin/api/$version/webhooks/949207433372.json");
        $hooks_tag = $sc->call('GET', "/admin/api/$version/webhooks.json");
        echo '<pre>';
        print_r($hooks_tag);
    }

}
